package net.jnjmx.ch11;

import java.util.SortedSet;
import java.util.TreeSet;
import java.lang.reflect.Constructor;
import javax.management.*;
import javax.management.modelmbean.*;
/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/** 
 * Example illustrating how to monitor an external UDDI registry which is owned
 * by another company using an MBean. 
 * - Finds the MBeanServer
 * - Creates a UDDIClientManager MBean for the IBM UDDI test site
 * - Registers the MBean
 * - Checks UDDI application availability 
 * - If its not available it checks network availability
 */

public class MonitorPartnerRegistry {

	public static void main(String[] args) {

		System.out.println("Running UDDI Registry Monitor");
		try {

			/** Find an MBeanServer for our MBean */
			MBeanServer mbs = MBeanServerFactory.createMBeanServer();
			System.out.println("created MBeanServer");

			/** Create a name for the MBean */
			ObjectName mbname = new ObjectName("webservices:id=UDDIClientView");
			System.out.println("mbean name " + mbname.toString());

			UDDIRegistryClientManager uddiMBean =
				new UDDIRegistryClientManager();

			// The following values point to the IBM UDDI test site
			uddiMBean.setURL(
				"http://www-3.ibm.com/services/uddi/testregistry/inquiryapi");
			// uddiMBean.setURL("https://www-3.ibm.com/services/uddi/testregistry/protect/publishapi");

			// This one works: The following values point to the IBM UDDI site
			//uddiMBean.setURL("http://www-3.ibm.com/services/uddi/inquiryapi");
			// uddiMBean.setURL("https://www-3.ibm.com/services/uddi/protect/publishapi");

			// This is still UDDI v1: The following values point to the Microsoft UDDI test site
			// uddiMBean.setURL("http://test.uddi.microsoft.com/inquire");
			// uddiMBean.setURL("https://test.uddi.microsoft.com/publish");

			// This one works: The following values point to the Microsoft UDDI official site
			// uddiMBean.setURL("http://uddi.microsoft.com/inquire");
			// uddiMBean.setURL("https://uddi.microsoft.com/publish");

			// The following values for a locally installed test server
			// uddiMBean.setURL("http://localhost:8080/services/uddi/servlet/uddi");
			// uddiMBean.setURL("http://localhost:8080/services/uddi/servlet/uddi");

			// uddiMBean.setURL("http://www-3.ibm.com/services/uddi/testregistry/inquiryapi");

			// ** register the ModelMBean that is now ready to run with the MBeanServer
			ObjectInstance registeredMBean =
				mbs.registerMBean((Object) uddiMBean, mbname);

			/* server stuff */

			//		ObjectName httpon = new ObjectName("adapters:id=Http");
			//		mbs.createMBean("com.tivoli.jmx.http_pa.Listener", httpon);
			//		mbs.invoke(httpon, "startListener", new Object[] {}, new String[] {}); 
			Long response =
				(Long) mbs.invoke(
							mbname,
							"checkNetworkAvailability",
							new Object[] {}, 
							new String[] {});
			if (response.longValue() <= 0) {
				System.out.println(
					"UDDI registry is not accessible through the network");
			} else {
				System.out.println(
					"UDDI registry is accessible through the network, "
						+ " response time is "
						+ response.toString()
						+ "ms checking application availability");
			}

			response =
				(Long) mbs.invoke(
					mbname,
					"checkAvailability",
					new Object[] { "I" },
					new String[] { "java.lang.String" });

			if (response.longValue() <= 0) {
				// if response is 0 or -1...
				System.out.println(
					"UDDI Registry is Not Available, checking network access");
					response = (Long) mbs.invoke(mbname, "checkNetworkAvailability", 
								new Object[] {}, 
								new String[] {});
			} else {
				// response is 1
				System.out.println(
					"UDDI Registry is Available, response time was "
						+ response.toString()
						+ "ms");
			}

		} catch (Exception e) {
			System.out.println("UDDI Monitor failed with " + e.getMessage());
		}
	}
}
