package net.jnjmx.ch11;


/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/** 
 * UDDIRegistryClient manages a UDDI registry as a client, not hosting
 * the UDDI registry or the implementation of the registry. It pings the UDDI
 * registry and sends a test find for any business starting with 'I'. 
 * 
 * This example requires installation of UDDI4J available from
 *  http://www.uddi4j.org or http://www.ibm.com/ 
 */

import org.uddi4j.*; 
import org.uddi4j.client.*;
import org.uddi4j.datatype.*;
import org.uddi4j.datatype.business.*;
import org.uddi4j.datatype.binding.*;
import org.uddi4j.request.*;
import org.uddi4j.response.*;
import org.uddi4j.transport.TransportFactory;
import org.uddi4j.util.*;
import org.w3c.dom.Element;
import java.util.Vector;
import java.util.Properties;
import java.io.*;

import java.net.*;


public class UDDIRegistryClient {

	String logonID = "";
	String logonPW = "";
	String regName = "";
	String regURL = "";
			
	/**
	 * Constructor for UDDIRegistryClient.
	 */
	public UDDIRegistryClient() {
	}
	
	/**
	 * Constructor for UDDIRegistryClient.
	 * @param String name of the UDDI registry
	 * @param String url inquirey URL for the UDDI registry
	 * @param String logonName logon name for a user account on the UDDI registry
	 * @param String pw password for a user account on the UDDI registry
	 * */
	public UDDIRegistryClient(String name, String url, String inLogonID, String inLogonPW) {
		System.out.println("instantiating UDDIRegistryClient");
		logonID = inLogonID;
		logonPW = inLogonPW;
		regName = name;
		regURL = url;

	}
	
	/**
	 * ping checks the network availability of a hostname
	 * >0 is good, 0 is bad, -1 is unknown host or other exception
	 * this ping uses the ping already on the system though the Runtime.
	 * This will work on a Windows 2000 system with default ping. 
	 * On a unix or using another ping tool, the parsing to extract
	 * the correct average response time my need to be customized.
	 * 
	 * @param String url to be pinged. The URL will be converted an IP address.
	 * @return long average response time
	 * */
	public long ping(String url) { 
		long roundTripTime = 0;
		long sendTime, endTime = 0;
		long timeMeasured = -1;
		long pingMS = 0;
		InetAddress uddiIPAddr;
		URL pingURL;
		String printstring;
		if (url.equals(""))  { url = regURL; }
		if (url.equals("")) {
			url = regURL;
			// search for all businesses starting with I ("IBM" should be there)
			url = "http://www-3.ibm.com/services/uddi/testregistry/inquiryapi";
		}
		System.out.println("UDDIRegistryClient:ping ping for url " + url);

		try {
			pingURL = new URL(url);
			String hostname = pingURL.getHost();	
			
			String th = (InetAddress.getLocalHost()).getHostName();
			
			uddiIPAddr = InetAddress.getByName(hostname);
			String hostipaddr = uddiIPAddr.getHostAddress();
			System.out.println("Pinging " + hostname + " at " + hostipaddr + " from " + th);
			String[] cmdString = new String[] {"ping", hostipaddr};
			
			endTime = -1;
			int avgindex, msindex;
			String avgms = null;
			
			sendTime = System.currentTimeMillis();
			Process pingproc = (Runtime.getRuntime()).exec(cmdString);
			InputStream pingos = pingproc.getInputStream();
			endTime = System.currentTimeMillis();
			BufferedReader rdr = new BufferedReader(new InputStreamReader(pingos));
			printstring = rdr.readLine();
			while ((printstring != null) && (avgms == null)) {
				// System.out.println("ping returned: " + printstring);
				avgindex = printstring.indexOf("Average");				// *Average
				if (avgindex > 0) {
					msindex = printstring.indexOf("ms", avgindex);		// *Average = xxxms*
					avgms = printstring.substring(avgindex+7, msindex); // * = xxx*
				}
				printstring = rdr.readLine();
			}
			if (avgms != null) {
				avgms =  avgms.replace('=', ' ');
				avgms = avgms.trim();          // should leave just the digits
				pingMS = (new Long(avgms)).longValue();
			}
			if (pingMS <= 0) {
				// setup url connnection, the url has been redirected to
				HttpURLConnection toUDDI = (HttpURLConnection) pingURL.openConnection();
				toUDDI.setUseCaches(false);
				toUDDI.setRequestMethod("POST");
				toUDDI.connect();
				InputStream is = toUDDI.getInputStream();
				BufferedReader urlrdr = new BufferedReader(new InputStreamReader(is));
				printstring = urlrdr.readLine();
				while (printstring != null)  {
					// System.out.println("url returned: " + printstring);
					printstring = urlrdr.readLine();
				}
			}
			
		} catch (UnknownHostException eu) {
			System.out.println("URL " + url + " is unknown");
			return -1;
		} catch (MalformedURLException em) {
			System.out.println("URL " + url + " is malformed");
			return -1;
		} catch (Exception e) { // failed
			System.out.println("Ping failed");
			return -1;
		};
		
		return pingMS;
	}
	
	/**
	 * test checks the UDDI registry application an ensures it is functional
	 * by sending a request to find businesses that start with "I". 
	 * >0 is good, 0 is bad, -1 is unknown host or other exception
	 * this ping uses the ping already on the system though the Runtime.
	 * This will work on a Windows 2000 system with default ping. 
	 * On a unix or using another ping tool, the parsing to extract
	 * the correct average response time my need to be customized.
	 * 
	 * @param String url of the UDDI server inquiry URL to be tested. 
	 * @param String businessName The name of the business or string
	 * to test for using a case sensitive match
	 * @return long response time
	 * */	
	public long test(String url, String businessName) { 
		System.setProperty(TransportFactory.PROPERTY_NAME, "org.uddi4j.transport.ApacheAxisTransport");
		UDDIProxy proxy;
		if (url.equals(""))  { url = regURL; }
		if (url.equals("")) {
			// search for all businesses starting with I ("IBM" should be there)
			url = "http://www-3.ibm.com/services/uddi/testregistry/inquiryapi";
		}
		if (businessName.equals("")) {
			// search for all businesses starting with I ("IBM" should be there)
			businessName = "I";
		}
		// System.out.println("UDDIRegistryClient:test for url " + url);
		
		try {
			proxy = new UDDIProxy();
			proxy.setInquiryURL(url);
		} catch (Exception e) {
		     e.printStackTrace();
			 return -1;
		}

		try {
			//creating vector of Name Object
         	Vector bizNames = new Vector();
         	bizNames.add(new Name(businessName));

	        // Setting FindQualifiers to 'caseSensitiveMatch'
	        FindQualifiers findQualifiers = new FindQualifiers();
        	Vector qualifier = new Vector();
        	qualifier.add(new FindQualifier("caseSensitiveMatch"));
            findQualifiers.setFindQualifierVector(qualifier);

			long findStartTime = System.currentTimeMillis();
			System.out.println("Looking for businesses " + businessName);
         	// Find businesses by name
         	// And setting the maximum rows to be returned as 5.
        	BusinessList businessList = proxy.find_business(bizNames, null, null, null, null, findQualifiers,5);
			// System.out.println("uddi getBusiness return " + businessList.getBusinessInfos().size());
			long findReturnTime = System.currentTimeMillis();

	        Vector businessInfoVector  = businessList.getBusinessInfos().getBusinessInfoVector();
//   	    for (int i = 0; i < businessInfoVector.size(); i++) {
//        			BusinessInfo businessInfo = (BusinessInfo)businessInfoVector.elementAt(i);
//   				Print name for each business
//        			System.out.println(businessInfo.getNameString());
//         	}
         	 if (businessInfoVector.size() > 0) {
         	 	long elapsedFindTime = findReturnTime - findStartTime;
				return elapsedFindTime;
			 }
			 else {
			 	return -1;
			 }
      	} catch (UDDIException e) {
      		 System.out.println("UDDIRegistryClient:test UDDIException " + e.getMessage());
	         DispositionReport dr = e.getDispositionReport();
	         if (dr!=null) {
	            System.out.println("UDDIException faultCode:" + e.getFaultCode() +
	                               "\n operator:" + dr.getOperator() +
	                               "\n generic:"  + dr.getGeneric() +
	                               "\n errno:"    + dr.getErrno() +
	                               "\n errCode:"  + dr.getErrCode() +
	                               "\n errInfoText:" + dr.getErrInfoText());
	         }
	         // e.printStackTrace();
			 return -1;
	    } catch (Exception e) {
     		 // e.printStackTrace();
     		 System.out.println("UDDIRegistryClient:test exception " + e.getMessage());
			 return -1;
	   	}
	}
}
