package net.jnjmx.ch11;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean that manages a UDDI Registry as a user of the registry.
 * It provides the URL of the UDDI registry, its status, its network availability.
 *
 * This class interacts with the UDDI Registry using UDDI4J
 *
 */
public class UDDIRegistryClientManager
	implements UDDIRegistryClientManagerMBean {
	
	String regName = "";
	String regURL = "";
	String logonID = "";
	String logonPW = "";
	String currentStatus = "available";
	UDDIRegistryClient uddiClient;
	
	/**
	 * Constructor for UDDIRegistryClientManager
	 * @param String name of the UDDI registry
	 * @param String url inquirey URL for the UDDI registry
	 * @param String logonName logon name for a user account on the UDDI registry
	 * @param String pw password for a user account on the UDDI registry
	 * */
	public UDDIRegistryClientManager(String name, String url, String logonName, String pw) {
		regName = name;
		regURL = url;
		logonID = logonName;
		logonPW = pw;
		uddiClient = new UDDIRegistryClient(regName, regURL, logonID, logonPW);
	}

	/**
	 * Constructor for UDDIRegistryClientManager.
	 */
	public UDDIRegistryClientManager() {
		super();
		uddiClient = new UDDIRegistryClient();
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#getName()
	 */
	public String getName() {
		return regName;
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#setName(String)
	 */
	public void setName(String name) {
		regName = name;
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#getURL()
	 */
	public String getURL() {
		return regURL;
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#setURL(String)
	 */
	public void setURL(String url) {
		regURL = url;
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#getStatus()
	 */
	public String getStatus() {
		return currentStatus;
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#setStatus(String)
	 */
	public void setStatus(String avail) {
		currentStatus = avail;
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#checkAvailability(int)
	 * It runs the UDIRegistryClient.test method
	 */
	public Long checkAvailability(String businessName) { 
		// System.out.println("UDDIRegistryClientManager:checkAvailability:testing uddi server " + regURL);
		long rc = uddiClient.test(regURL, businessName);
		if (rc > 0)
			currentStatus = "available";
		else 
			currentStatus = "not available";
		return new Long(rc);
	}

	/**
	 * @see jnjmxch11.UDDIRegistryClientManagerMBean#checkNetworkAvailability()
	 * 0 = not available, else returns response time for ping
	 * Note, this method will not work if the UDDI Registries URL has been redirected.
	 * Executes the UDDIRegistryClient.ping method
	 */
	public Long checkNetworkAvailability() {
		// 0 = not available, else returns response time for ping
		// Note, this method will not work if the UDDI Registries URL
		//       has been redirected.
		// System.out.println("UDDIRegistryClientManager:checkNetworkAvailability:pinging uddi server for " + regURL);
		long rc = 0;
		Long zero = new Long(0);
		try {
			rc = uddiClient.ping(regURL);	
		}
		catch (Exception e) {
			return zero;
		}
		if (rc > 0 ) {
			currentStatus = "available";
			return new Long(rc);
		}
		else {
			currentStatus = "not available";
			return zero;
		}	
	}

}
