package net.jnjmx.ch11;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 * */

/** 
 *
 * UDDIRegistryData would be a private interface to a UDDI registry
 * that you own, host, or implement and need to control.
 * For the examples here, these methods are empty since we do not
 * own a UDDI registry and this is illustrative.
 * 
 * The methods in this class retrieve metrics from and about the 
 * UDDI registry. They will create read only attributes in an MBean.
 * 
 * This example requires installation of UDDI4J available from
 *  http://www.uddi4j.org or http://www.ibm.com/
 */

import org.uddi4j.*;
import org.uddi4j.client.*;
import org.uddi4j.datatype.business.*;
import org.uddi4j.response.*;
import org.uddi4j.util.*;
import org.w3c.dom.Element;
import java.util.Vector;
import java.util.Properties;

public class UDDIRegistryData {

	String logonID = "";
	String logonPW = "";
	String regURL = "";
	String regName = "";
	int regEntries, regAvgFind, regAvgGet, regHiFind = 0;
	int regHiGet, regLoFind, regLoGet = 0;
	int regGets=0, regSearchs=0;
	String version = "";
	
	/**
	 * Constructor for UDDIRegistryData.
	 */
	public UDDIRegistryData() { 
	}
	
	/**
	 * Constructor for UDDIRegistryClient.
	 * @param String name of the UDDI registry
	 * @param String url inquirey URL for the UDDI registry
	 * @param String logonName logon name for a user account on the UDDI registry
	 * @param String pw password for a user account on the UDDI registry
	 */
	public UDDIRegistryData(String name, String url, String inLogonID, String inLogonPW) {
		logonID = inLogonID;
		logonPW = inLogonPW;
		regName = name;
		regURL = url;
	}
	
	/**
	 * getVersion returns the version number of the UDDI Registry in VnRn.n format
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return String version in VnRn.n format
	 * */ 
	public String getVersion() {
		return version;
	}
	
	/**
	 * getNumberOfEntries returns the number of entries in the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int number of entries
	 * */ 
	public int getNumberOfEntries() {
		return regEntries;
	}
	
	/**
	 * getAverageFindTime returns the average time for find 
	 * operations on the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int average time for performing find operations
	 * */
	public int getAverageFindTime() {
		return regAvgFind;
	}
	
	/**
	 * getAverageGetTime returns the average time for get
	 * operations on the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int average time for performing get operations
	 * */
	public int getAverageGetTime() {
		return regAvgGet;
	}
	
	/**
	 * getHighestFindTime returns the longest time for a find 
	 * operation on the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int longest time for performing find operations
	 * */
	public int getHighestFindTime() {
		return regHiFind;
	}
	
	/**
	 * getHighestGetTime returns the longest time for a get 
	 * operation on the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int longest time for performing get operations
	 * */
	public int getHighestGetTime() {
		return regHiGet;
	}
	
	/**
	 * getLowestFindTime returns the fastest time for a find 
	 * operation on the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int fastest time for performing find operations
	 * */
	public int getLowestFindTime() {
		return regLoFind;
	}
	
	/**
	 * getLowestGetTime returns the fastest time for a find 
	 * operation on the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int fastest time for performing get operations
	 * */
	public int getLowestGetTime() {
		return regLoGet;
	}
	
	/**
	 * getNumberOfGets returns the number of get
	 * operations processed by the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int number of get operations
	 * */
	public	int getNumberOfGets() {
		return regGets;
	}
	
	/**
	 * getNumberOfSearchs returns the number of find
	 * operations processed by the UDDI Registry
	 * Each of these methods would invoke a UDDI Registries MBean or a 
	 * private api with the UDDI Registry to get the statistic.
	 * @return int number of find operations
	 * */
	public int getNumberOfSearchs() {
		return regSearchs;
	}

}
