package net.jnjmx.ch11;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean implementation to manage a UDDI Registry as the owner of the 
 * registry. It supports checking availability, attributes, and operations.
 * */
public class UDDIRegistryManager implements UDDIRegistryManagerMBean {
	String regName = "";
	String regURL = "";
	String logonID = "";
	String logonPW = "";
	String regVersion = "";
	String queryURL = "";
	String publishURL = "";
	String currentStatus = "unknown";
	int regEntries, regAvgFind, regAvgGet = 0;
	int regHiFind, regHiGet, regLoFind, regLoGet = 0;
	int regGets = 0, regSearchs = 0;
	UDDIRegistryClient uddiClient;
	UDDIRegistryController uddiController;
	UDDIRegistryData uddiData;

	/**
	 * Constructor for UDDIRegistryManager.
	 */
	UDDIRegistryManager() {
		uddiClient = new UDDIRegistryClient();
		uddiController = new UDDIRegistryController();
		uddiData = new UDDIRegistryData();
		regName = "UDDIRegistry";
	}
	
	/**
	 * Constructor for UDDIRegistryManager.
	 * @param String name of the UDDI registry
	 * @param String url inquirey URL for the UDDI registry
	 * @param String logonName logon name for a user account on the UDDI registry
	 * @param String pw password for a user account on the UDDI registry
	 * */	
	UDDIRegistryManager(String name, String url, String logonName, String pw) {
		regName = name;
		regURL = url;
		logonID = logonName;
		logonPW = pw;
		uddiClient = new UDDIRegistryClient(regName, regURL, logonID, logonPW);
		uddiController = new UDDIRegistryController(regName, regURL, logonID, logonPW);
		uddiData = new UDDIRegistryData();
	}
	// identity
	/** see UDDIRegistryClientManagerMBean 	 * */
	public String getName() { 
		return regName;
	}
	
	/** see UDDIRegistryClientManagerMBean 	 * */
	public void setName(String name) {
		regName = name;
	}
	
	/** see UDDIRegistryClientManagerMBean 	 * */
	public String getVersion() {
		regVersion = uddiData.getVersion();
		return regVersion;
	}
	
	// config
	
	/** see UDDIRegistryClientManagerMBean 	 * */
	public String getURL() {
		// this is the inquiry url that does not require a userid and pw
		return regURL;
	}
	
	/** see UDDIRegistryClientManagerMBean 	 * */	
	public void setURL(String url) {
		regURL = url;
	}
	
	/** see UDDIRegistryClientManagerMBean 	 * */	
	public String[] getURLs() {
		// this is the inquiry and publish url that does require https and maybe a userid and pw
		return new String[] {queryURL, publishURL};
	}
	
	/** see UDDIRegistryClientManagerMBean 	 * */	
	public void setURLs(String[] secureURLs) {
		queryURL = secureURLs[1];
		publishURL = secureURLs[2];
	}
	
	/** see UDDIRegistryClientManagerMBean 	 * */	
	public String getStatus() {
		long rca = uddiClient.test(regURL, "I");
		if (rca >= 0) {
			currentStatus = "registry up";
		} else { 
			long rcb = uddiClient.ping(regURL);
			if (rcb > 0) {
				currentStatus = "network ok, registry down";
			} else {
				currentStatus = "network down";
			}
		}
		return currentStatus;
	}
	
	// metrics
	/** * see UDDIRegistryData	 * */
	public int getNumberOfEntries() {
		regEntries = uddiData.getNumberOfEntries();
		return regEntries;
	}
	
	/** * see UDDIRegistryData	 * */
	public int getNumberOfGets() {
		regGets = uddiData.getNumberOfGets();
		return regGets;
	}
	
	/** * see UDDIRegistryData	 * */
	public int getNumberOfSearchs() {
		regSearchs = uddiData.getNumberOfSearchs();
		return regSearchs;
	}
	
	/** * see UDDIRegistryData	 * */
	public int getAverageSearchTime() {
		regAvgFind = uddiData.getAverageFindTime();
		return regAvgFind;
	}
	
	/** * see UDDIRegistryData	 * */
	public int getAverageGetTime() {
		regAvgGet = uddiData.getAverageGetTime();
		return regAvgGet;
	}
	
	/** * see UDDIRegistryData	 * */
	public int getHighSearchTime() {
		regHiFind = uddiData.getHighestFindTime();
		return regHiFind;
	}
	
	/** * see UDDIRegistryData	 * */
	public int getHighGetTime() {
		regHiGet = uddiData.getHighestGetTime();
		return regHiGet;
	}
	
		
	/** * see UDDIRegistryData	 * */
	public int getLowSearchTime() {
		regLoFind = uddiData.getLowestFindTime();
		return regLoFind;
	}
	
		
	/** * see UDDIRegistryData	 * */
	public int getLowGetTime() {
		regLoGet = uddiData.getLowestGetTime();
		return regLoGet;
	}
	
	// operations
		
	/** * see UDDIRegistryController	 * */
	public int start() {
		int rc = uddiController.start();
		return rc;
	}
	
	/** * see UDDIRegistryController	 * */
	public int stop() {
		int rc = uddiController.stop();
		return rc;
	}
	
	/** * see UDDIRegistryController	 * */
	public int backUpRegistryData() {
		int rc = uddiController.backup();
		return rc;
	}
	
}
