package net.jnjmx.ch11;

/**
 * @author - Heather Kreger
 * Java and JMX: Building Manageable Systems, Chapter 11
 * 
 * The program is provided "as is" without any warranty express or implied,
 * including the warranty of non-infringement and the implied warranties of
 * merchantibility and fitness for a particular purpose. The author will not be
 * liable for any damages suffered by you or any third party claim against 
 * you regarding the Program.
 */
/**
 * Standard MBean implementation to manage a UDDI Registry as the owner of the 
 * registry. It supports checking availability, attributes, and operations.
 * */
public class UDDIRegistryRuntimeManager
	implements UDDIRegistryRuntimeManagerMBean {

	String regName = "";
	String regURL = "";
	String logonID = "";
	String logonPW = "";
	String regVersion = "";
	String queryURL = "";
	String publishURL = "";
	String currentStatus = "up";
	int regEntries, regAvgFind, regAvgGet, regGets, regSearchs = 0;
	int regHiFind, regHiGet, regLoFind, regLoGet = 0;
	UDDIRegistryClient uddiClient;
	UDDIRegistryController uddiController;
	UDDIRegistryData uddiData;
	
	/**
	 * Constructor for UDDIRegistryRuntimeManager.
	 */
	public UDDIRegistryRuntimeManager() {
		super();
		uddiClient = new UDDIRegistryClient();
		uddiController = new UDDIRegistryController();
		uddiData = new UDDIRegistryData();	
		regName = "UDDI Registry";
	}
	
	/**
	 * Constructor for UDDIRegistryRuntimeManager.
	 * @param String name of the UDDI registry
	 * @param String url inquirey URL for the UDDI registry
	 * @param String logonName logon name for a user account on the UDDI registry
	 * @param String pw password for a user account on the UDDI registry
	 * */	
	UDDIRegistryRuntimeManager(String name, String url, String logonName, String pw) {
		regName = name;
		regURL = url;
		logonID = logonName;
		logonPW = pw;
		uddiClient = new UDDIRegistryClient(regName, regURL, logonID, logonPW);
		uddiController = new UDDIRegistryController(regName, regURL, logonID, logonPW);
		uddiData = new UDDIRegistryData(regName, regURL, regName, regURL);
	}
	
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getName()
	 */
	public String getName() {
		return regName;
	}

	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#setName(String)
	 */
	public void setName(String name) {
		regName = name;
	}

	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getVersion()
	 */
	public String getVersion() {
		regVersion = uddiData.getVersion();
		return regVersion;
	}

	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getIPAddr()
	 */
	public String getURL() {
		return regURL;
	}

	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#setIPAddr(String)
	 */
	public void setURL(String url) {
		regURL = url;
	}

	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getURLs()
	 */
	public String[] getURLs() {
		return new String[] {queryURL, publishURL};
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getStatus()
	 */
	public String getStatus() {
		long rca = uddiClient.test(regURL, "I");
		if (rca > 0) {
			currentStatus = "registry up";
		} else { 
			long rcb = uddiClient.ping(regURL);
			if (rcb == 0) {
				currentStatus = "registry down";
			} else {
				currentStatus = "network down";
			}
		}
		return currentStatus;
	}
	
	// metrics
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getNumberOfEntries()
	 */
	public int getNumberOfEntries() {
		regEntries = uddiData.getNumberOfEntries();
		return regEntries;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getNumberOfGets()
	 */
	public int getNumberOfGets() {
		regGets = uddiData.getNumberOfGets();
		return regGets;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getNumberOfSearchs()
	 */
	public int getNumberOfSearchs() {
		regSearchs = uddiData.getNumberOfSearchs();
		return regSearchs;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getAverageSearchTime()
	 */
	public int getAverageSearchTime() {
		regAvgFind = uddiData.getAverageFindTime();
		return regAvgFind;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getAverageGetTime()
	 */
	public int getAverageGetTime() {
		regAvgGet = uddiData.getAverageGetTime();
		return regAvgGet;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getHighSearchTime()
	 */
	public int getHighSearchTime() {
		regHiFind = uddiData.getHighestFindTime();
		return regHiFind;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getHighGetTime()
	 */
	public int getHighGetTime() {
		regHiGet = uddiData.getHighestGetTime();
		return regHiGet;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getLowSearchTime()
	 */
	public int getLowSearchTime() {
		regLoFind = uddiData.getLowestFindTime();
		return regLoFind;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#getLowGetTime()
	 */
	public int getLowGetTime() {
		regLoGet = uddiData.getLowestGetTime();
		return regLoGet;
	}
	
	// operations
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#start()
	 */
	public int start() {
		int rc = uddiController.start();
		return rc;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#stop()
	 */
	public int stop() {
		int rc = uddiController.stop();
		return rc;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#backUpRegistryData()
	 */
	public int backUpRegistryData() {
		int rc = uddiController.backup();
		return rc;
	}

	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#setStatus()
	 */
	public void setStatus(String avail) {
		currentStatus = avail;
	}
	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#checkAvailability()
	 */
	public long checkAvailability(String businessName) {
		System.out.println("testing uddi server");
		long rc = uddiClient.test(regURL, businessName);
		if (rc > 0)
			currentStatus = "available";
		else 
			currentStatus = "not available";
		return rc;
	}

	/**
	 * @see net.jnjmx.ch11.UDDIRegistryRuntimeManagerMBean#checkNetworkAvailability()
	 */
	public long checkNetworkAvailability() {
		System.out.println("pinging uddi server");
		long rc = 1;
		try {
			rc = uddiClient.ping(regURL);	
		}
		catch (Exception e) {
			rc = -1;
		}
		if (rc > 0) {
			currentStatus = "available";
			return rc;
		}
		else {
			currentStatus = "not available";	
			return 0;
		}
	}


}
