package net.jnjmx.ch4;

/**
 * @author Heather Kreger
 * Chapter 4:  Java and JMX: Building Manageable Systems
 * ApacheServer: interacts with an Apache server through a URL
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.StringTokenizer;
import javax.management.*;

public class ApacheServer {
	// String constants for each of the Apache-related attributes
	private static final String BUSY_WORKERS = "BusyWorkers";
	private static final String BYTES_PER_REQ = "BytesPerReq";
	private static final String BYTES_PER_SEC = "BytesPerSec";
	private static final String CPU_LOAD = "CPULoad";
	private static final String IDLE_WORKERS = "IdleWorkers";
	private static final String REQ_PER_SEC = "ReqPerSec";
	private static final String SCOREBOARD = "Scoreboard";
	private static final String TOTAL_ACCESSES = "Total Accesses";
	private static final String TOTAL_KBYTES = "Total kBytes";
	private static final String UPTIME = "Uptime";

	private static final int MAX_FAILURES = 3;

	// state constants for the Apache Server (brought over from ApapcheMBean interface
	public String DOWN = "DOWN";
	public String NOT_RESPONDING = "NOT_RESPONDING";
	public String RUNNING = "RUNNING";
	public String UNKNOWN = "UNKNOWN";

	private int failures = 0;
	private String state = UNKNOWN;
	private long tzero;
	private URL url;

	// constructor which accepts the URL for the Apache Server to be managed
	public ApacheServer(String url)
		throws MalformedURLException, IllegalArgumentException {
		setServer(url);
		// test the ability to communicate using the URL
		try {
			getValue(UPTIME);
		} catch (Exception e) {
			System.out.println(
				"Apache server at " + url + " does not respond.");
		}
	}

	// return the url of the Apache server being managed
	// Server is a readable and writeable attribute because
	// it has a getter and setter.
	public String getServer() {
		return this.url.toString();
	}

	// set the url of the Apache server being managed. 
	public void setServer(String url)
		throws MalformedURLException, IllegalArgumentException {
		this.url = new URL(url);
		// test to be sure the URL is really an Apache server URL
		if (isStatusUrl(this.url) == false) {
			throw new IllegalArgumentException(url.toString());
		}
		this.state = ApacheMBean.UNKNOWN;
	}

	// return the current state of the Apache server
	public String getState() {
		return this.state;
	}

	// Example operation for start
	// If this were running in the same JVM, this is
	// where the apache process initialzation would go
	public String start() {
		this.state = RUNNING;
		return this.state;
	}

	// Example operation for stop
	// If this were running in the same JVM, this is
	// where the apache process initialzation would go
	public String stop() {
		this.state = DOWN;
		return this.state;
	}

	// Validate that the url is an Apache server status url
	private boolean isStatusUrl(URL url) {
		return url.toString().endsWith("server-status?auto");
	}

	// Get methods for metrics retrieved via the Apache url
	// These are read only attributes as no 'setters' are 
	// defined for them

	public int getBusyWorkers() throws ApacheMBeanException {
		return getIntValue(BUSY_WORKERS);
	}

	public int getBytesPerSec() throws ApacheMBeanException {
		return getIntValue(BYTES_PER_SEC);
	}

	public float getBytesPerReq() throws ApacheMBeanException {
		return getFloatValue(BYTES_PER_REQ);
	}

	public float getCpuLoad() throws ApacheMBeanException {
		return getFloatValue(CPU_LOAD);
	}

	public int getIdleWorkers() throws ApacheMBeanException {
		return getIntValue(IDLE_WORKERS);
	}

	public float getReqPerSec() throws ApacheMBeanException {
		return getFloatValue(REQ_PER_SEC);
	}

	public String getScoreboard() throws ApacheMBeanException {
		return getStringValue(SCOREBOARD);
	}

	public int getTotalAccesses() throws ApacheMBeanException {
		return getIntValue(TOTAL_ACCESSES);
	}

	public long getTotalKBytes() throws ApacheMBeanException {
		return getLongValue(TOTAL_KBYTES);
	}

	public long getUptime() throws ApacheMBeanException {
		return getLongValue(UPTIME);
	}

	// These methods are used as internal utilities by the getter methods
	// There is one for each type returning the base Java type

	private String getValue(String value)
		throws ApacheMBeanException, NoSuchFieldException, IOException {
		String result;
		URLConnection k = establishConnection();

		result = (String) readStatus(k, value);

		if (result == null) {
			throw new NoSuchFieldException(value);
		}
		return result;
	}

	private float getFloatValue(String value) throws ApacheMBeanException {
		float result;
		try {
			result = Float.parseFloat((String) getValue(value));
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	private int getIntValue(String value) throws ApacheMBeanException {
		int result;
		try {
			result = Integer.parseInt((String) getValue(value));
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	private long getLongValue(String value) throws ApacheMBeanException {
		long result;
		try {
			result = Long.parseLong((String) getValue(value));
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	private String getStringValue(String value) throws ApacheMBeanException {
		String result;
		try {
			result = (String) getValue(value);
		} catch (IOException x) {
			throw new ApacheMBeanException(x);
		} catch (NoSuchFieldException x) {
			throw new ApacheMBeanException(x);
		}
		return result;
	}

	// Establishes the connection to the URL
	// If the connection fails, then the state
	// is changed to DOWN or NOT_RESPONDING

	private URLConnection establishConnection()
		throws IOException, ApacheMBeanException {
		URLConnection k = this.url.openConnection();
		try {
			k.connect();
			this.failures = 0;
			this.state = ApacheMBean.RUNNING;
		} catch (IOException x) {
			if (++this.failures > MAX_FAILURES) {
				this.state = DOWN;
			} else {
				this.state = NOT_RESPONDING;
			}
			throw new ApacheMBeanException("state: " + this.state);
		}
		return k;
	}

	// Parses the data returned from the URL

	private String readStatus(URLConnection k, String value)
		throws IOException {
		BufferedReader r =
			new BufferedReader(new InputStreamReader(k.getInputStream()));
		for (String l = r.readLine(); l != null; l = r.readLine()) {
			StringTokenizer st = new StringTokenizer(l, ":");
			if (st.nextToken().trim().equals(value)) {
				// if its the right value
				return (String) (st.nextToken().trim()); // get wanted value
			} else {
				st.nextToken(); // past unwanted value
			}
		}
		return "";
	}

}
