package net.jnjmx.ch4;

import javax.management.*;
import javax.management.modelmbean.*;

import java.lang.reflect.Constructor;
import java.util.ArrayList;

/**
 * @author Heather Kreger
 * Chapter 4:  Java and JMX Designing Manageable Systems
 * ApacheServerManager: Manages the ApacheServer as a managed
 * resource using a model MBean. 
 * 
 * ModelMBeanInfo is created using ModelMBeanInfo APIs
 * rather than an XML file.
 */

public class ApacheServerManager {

	public static void main(String[] args) {
		boolean tracing = false;
		String serverURL = "";
		try {

			if (args.length != 0) {
				System.out.println(args[0]);
				if (args[0] != null)
					serverURL = serverURL.concat(args[0]);
				else
					serverURL = "http://www.apache.org/server-status?auto";
			}

			if (!serverURL.startsWith("http"))
				serverURL = "http://www.apache.org/server-status?auto";

			/** Find an MBeanServer for our MBean */
			MBeanServer myMBS = MBeanServerFactory.createMBeanServer();

			/** Instantiate an unconfigured RequiredModelMBean in the MBeanServer */
			RequiredModelMBean apacheMMBean =
				(RequiredModelMBean) myMBS.instantiate(
					"javax.management.modelmbean.RequiredModelMBean");

			/** Create a name for the MBean 
			 * domain = apacheManager
			 * one attribute: id = ApacheServer*/
			ObjectName apacheMBeanName =
				new ObjectName("apacheManager: id=ApacheServer");

			/** instantiate the ApacheServer utility to be delegated to */
			net.jnjmx.ch4.ApacheServer myApacheServer =
				new net.jnjmx.ch4.ApacheServer(serverURL);

			/** Set the configuration of the ModelMBean through the ModelMBeanInfo
			 ** This method is below */
			ApacheServerManager asmgr = new ApacheServerManager();
			ModelMBeanInfo apacheMMBeanInfo =
				asmgr.createApacheModelMBeanInfo(
					myApacheServer,
					apacheMBeanName);
			apacheMMBean.setModelMBeanInfo(apacheMMBeanInfo);

			// ** Set the ApacheServer as the managed resource
			apacheMMBean.setManagedResource(myApacheServer, "ObjectReference");

			// ** register the ModelMBean that is now ready to run with the MBeanServer
			ObjectInstance registeredApacheMMBean =
				myMBS.registerMBean((Object) apacheMMBean, apacheMBeanName);

			// ** Use the ModelMBean to send the notification into JMX where it gets sent to a manager 
			// adapter that is registered for notifications from the Apache MBeans and sends them to a pager system

			String apacheStatus =
				(String) apacheMMBean.invoke("getState", null, null);
				
			int apacheIW =
				((Integer) apacheMMBean.getAttribute("IdleWorkers")).intValue();	

			if ((apacheStatus.equals("DOWN"))
				|| (apacheStatus.equals("NOT_RESPONDING"))) {
				System.out.println("Apache Server is Down");
				apacheMMBean.sendNotification("Apache Server is Down");
				/*  You could create monitor to notify when up 
				 * again so web traffic can be routed back to it */
			} else
				System.out.println("Apache Server is " + apacheStatus);
		} catch (Exception e) {
			System.out.println(
				"Apache Server Manager failed with " + e.getMessage());
		}
		System.exit(0);
	}

	// Create the ModelMBeanInfo for the Apache server. 

	ModelMBeanInfo createApacheModelMBeanInfo(
		ApacheServer managedApache,
		ObjectName apacheMBeanName) {

		// Set the ApacheServer's class name, there are other
		// more flexible ways to do this.	
		Class apacheClass = null;
		try {
			apacheClass = Class.forName("net.jnjmx.ch4.ApacheServer");
		} catch (Exception e) {
			System.out.println("ApacheServer Class not found");
		}

		// Set the MBean's descriptor with default policies
		// MBean name is apacheMBeanName
		// logging notifications to jmxmain.log
		// caching attributes for 10 seconds
		Descriptor apacheDescription =
			new DescriptorSupport(
				new String[] {
					("name=" + apacheMBeanName),
					"descriptorType=mbean",
					("displayName=ApacheServerManager"),
					"type=net.jnjmx.ch4.ApacheServer",
					"log=T",
					"logFile=jmxmain.log",
					"currencyTimeLimit=10" });

		// Define Attributes in ModelMBean AttributeInfos

		ModelMBeanAttributeInfo[] apacheAttributes =
			new ModelMBeanAttributeInfo[11];

		// Declare BusyWorkers attribute
		// cache BusyWorkers for 3 seconds, 
		// use get method		
		Descriptor busyWorkersDesc =
			new DescriptorSupport(
				new String[] {
					"name=BusyWorkers",
					"descriptorType=attribute",
					"displayName=Apache BusyWorkers",
					"getMethod=getBusyWorkers",
					"currencyTimeLimit=3" });

		apacheAttributes[0] =
			new ModelMBeanAttributeInfo(
				"BusyWorkers",
				"int",
				"Apache Server Busy Workers",
				true,
				false,
				false,
				busyWorkersDesc);

		// Declare BytesPerSec attribute
		// Cache ByetesperSec for 10 seconds			
		Descriptor bytesPerSecDesc =
			new DescriptorSupport(
				new String[] {
					"name=BytesPerSec",
					"descriptorType=attribute",
					"displayName=Apache BytesPerSec",
					"getMethod=getBytesPerSec",
					"currencyTimeLimit=10" });

		apacheAttributes[1] =
			new ModelMBeanAttributeInfo(
				"BytesPerSec",
				"int",
				"Apache Server Bytes Per Sec",
				true,
				false,
				false,
				bytesPerSecDesc);

		// Declare BytesPerReq attribute			
		Descriptor bytesPerReqDesc =
			new DescriptorSupport(
				new String[] {
					"name=BytesPerReq",
					"descriptorType=attribute",
					"displayName=Apache BytesPerReq",
					"getMethod=getBytesPerReq",
					"currencyTimeLimit=10" });

		apacheAttributes[2] =
			new ModelMBeanAttributeInfo(
				"BytesPerReq",
				"int",
				"Apache Server Bytes Per Request",
				true,
				false,
				false,
				bytesPerReqDesc);

		// Declare IdleWorkers attribute			
		Descriptor idleWorkersDesc =
			new DescriptorSupport(
				new String[] {
					"name=IdleWorkers",
					"descriptorType=attribute",
					"displayName=Apache IdleWorkers",
					"getMethod=getIdleWorkers",
					"currencyTimeLimit=10" });

		apacheAttributes[3] =
			new ModelMBeanAttributeInfo(
				"IdleWorkers",
				"int",
				"Apache Server Idle Workers",
				true,
				false,
				false,
				idleWorkersDesc);

		// Declare ReqPerSec attribute			
		Descriptor reqPerSecDesc =
			new DescriptorSupport(
				new String[] {
					"name=ReqPerSec",
					"descriptorType=attribute",
					"displayName=Apache ReqPerSec",
					"getMethod=getReqPerSec",
					"currencyTimeLimit=5" });

		apacheAttributes[4] =
			new ModelMBeanAttributeInfo(
				"ReqPerSec",
				"int",
				"Apache Server Requests Per Second",
				true,
				false,
				false,
				reqPerSecDesc);

		// Declare Scoreboard attribute			
		Descriptor ScoreboardDesc =
			new DescriptorSupport(
				new String[] {
					"name=Scoreboard",
					"descriptorType=attribute",
					"displayName=Apache Scoreboard",
					"getMethod=getScoreboard",
					"currencyTimeLimit=10" });

		apacheAttributes[5] =
			new ModelMBeanAttributeInfo(
				"Scoreboard",
				"java.lang.String",
				"Apache Server Scoreboard",
				true,
				false,
				false,
				ScoreboardDesc);

		// Declare TotalAccesses attribute	
		// Do not Cache TotalAccesses		
		Descriptor totalAccessesDesc =
			new DescriptorSupport(
				new String[] {
					"name=TotalAccesses",
					"descriptorType=attribute",
					"displayName=Apache TotalAccesses",
					"getMethod=getTotalAccesses",
					"currencyTimeLimit=-1" });

		apacheAttributes[6] =
			new ModelMBeanAttributeInfo(
				"TotalAccesses",
				"int",
				"Apache Server total accesses",
				true,
				false,
				false,
				totalAccessesDesc);

		// Declare TotalKBytes attribute
		// Do not cache TotalKBytes			
		Descriptor totalKBytesDesc =
			new DescriptorSupport(
				new String[] {
					"name=TotalKBytes",
					"descriptorType=attribute",
					"displayName=Apache TotalKBytes",
					"getMethod=getTotalKBytes",
					"currencyTimeLimit=-1" });

		apacheAttributes[7] =
			new ModelMBeanAttributeInfo(
				"TotalKBytes",
				"int",
				"Apache Server total KiloBytes",
				true,
				false,
				false,
				totalKBytesDesc);

		// Declare Uptime attribute			
		Descriptor upTimeDesc =
			new DescriptorSupport(
				new String[] {
					"name=Uptime",
					"descriptorType=attribute",
					"displayName=Apache Uptime",
					"getMethod=getUptime",
					"currencyTimeLimit=10" });

		apacheAttributes[8] =
			new ModelMBeanAttributeInfo(
				"Uptime",
				"java.lang.String",
				"Apache Server Up Time",
				true,
				false,
				false,
				upTimeDesc);

		// Declare State attribute	
		// State has a getMethod		
		Descriptor stateDesc =
			new DescriptorSupport(
				new String[] {
					"name=State",
					"descriptorType=attribute",
					"displayName=Apache State",
					"getMethod=getState",
					"currencyTimeLimit=10" });

		apacheAttributes[9] =
			new ModelMBeanAttributeInfo(
				"State",
				"java.lang.String",
				"Apache Server state",
				true,
				false,
				false,
				stateDesc);

		// Declare Server attribute		
		// Server has a getMethod and a setMethod	
		Descriptor serverDesc =
			new DescriptorSupport(
				new String[] {
					"name=Server",
					"descriptorType=attribute",
					"displayName=Apache Server URL",
					"getMethod=getServer",
					"setMethod=setServer",
					"currencyTimeLimit=10" });

			apacheAttributes[10] =
				new ModelMBeanAttributeInfo(
					"Server",
					"java.lang.String",
					"Apache Server Busy Workers",
					true,
					true,
					false, 
					serverDesc);

		// Declare constructors for the managed resource	
		// One constructor which accepts one parameter, 
		// A String URL	
		Constructor[] myConstructors = apacheClass.getConstructors();
		
		ModelMBeanConstructorInfo[] apacheConstructors =
			new ModelMBeanConstructorInfo[1];

		MBeanParameterInfo[] constructorParms =
			new MBeanParameterInfo[] {
				(new MBeanParameterInfo("serverURL",
				"java.lang.String",
				"Apache Server URL"))};

		Descriptor apacheBeanDesc =
			new DescriptorSupport(
				new String[] {
					"name=ApacheServer",
					"descriptorType=operation",
					"role=constructor" });

		apacheConstructors[0] =
			new ModelMBeanConstructorInfo(
				"Apache",
				"ApacheServer(): Constructs a ApacheServer utility class",
				constructorParms,
				apacheBeanDesc);

		// Define Operations in ModelMBeanOperationInfos
		/* Operations: getBusyWorkers, getBytesPerSec, getBytesPerReq, 
			* getIdleWorkers, getReqPerSec, getScoreboard, getTotalAccesses, 
			* getTotalKBytes, getUptime are satisfied by getValue, getState, 
			* getServer, setServer(String URL), start, stop have own operations */

		// Declare getValue Operation String getValue()	
		// Set paramater array	
		ModelMBeanOperationInfo[] apacheOperations =
			new ModelMBeanOperationInfo[7];

		MBeanParameterInfo[] getParms = new MBeanParameterInfo[0];

		MBeanParameterInfo[] getValueParms =
			new MBeanParameterInfo[] {
				(new MBeanParameterInfo("FieldName",
				"java.lang.String",
				"Apache status field name"))};

		Descriptor getValueDesc =
			new DescriptorSupport(
				new String[] {
					"name=getValue",
					"descriptorType=operation",
					"class=ApacheServer",
					"role=operation" });

		apacheOperations[0] =
			new ModelMBeanOperationInfo(
				"getValue",
				"getValue(): get an apache status field",
				getValueParms,
				"java.lang.String",
				MBeanOperationInfo.INFO,
				getValueDesc);

		Descriptor getStateDesc =
			new DescriptorSupport(
				new String[] {
				"name=getState",
				"descriptorType=operation",
				"class=ApacheServer",
				"role=operation" });


		apacheOperations[1] =
			new ModelMBeanOperationInfo(
				"getState",
				"getState(): current status of apache server",
				getParms,
				"java.lang.String",
				MBeanOperationInfo.INFO,
				getStateDesc);

		Descriptor getServerDesc =
			new DescriptorSupport(new String[] { "name=getServer",
	 			"descriptorType=operation", 
	 			"class=ApacheServer",
	 			"role=operation" });

		apacheOperations[2] =
			new ModelMBeanOperationInfo("getServer",  
				"getServer(): URL of apache server",  
				getParms, 
				"java.lang.Integer", 
				MBeanOperationInfo.INFO,
				getServerDesc); 
	
		MBeanParameterInfo[] setParms =
			new MBeanParameterInfo[] {
				(new MBeanParameterInfo("url",
				"java.lang.String",
				"Apache Server URL"))};

		Descriptor setServerDesc =
			new DescriptorSupport(new String[] { "name=getServer",
	 			"descriptorType=operation", 
	 			"class=ApacheServer",
	 			"role=operation" }); 

		apacheOperations[3] =
			new ModelMBeanOperationInfo("setServer",  
				"getServer(): URL of apache server",  
				setParms, 
				"java.lang.String", 
				MBeanOperationInfo.ACTION, 
				setServerDesc); 

		MBeanParameterInfo[] startParms = new MBeanParameterInfo[0];

		Descriptor startDesc =
			new DescriptorSupport(new String[] { "name=start",
				"descriptorType=operation", 
				"class=ApacheServer",
				"role=operation" }); 

		apacheOperations[4] = new ModelMBeanOperationInfo("start",  
			"start(): start apache server",  
			startParms, 
			"java.lang.Integer", 
			MBeanOperationInfo.ACTION,
			startDesc); 

		MBeanParameterInfo[] stopParms = new MBeanParameterInfo[0];
		Descriptor stopDesc = new DescriptorSupport(new String[] { "name=stop",
			 "descriptorType=operation", 
			"class=ApacheServer",
			 "role=operation" }); 

		apacheOperations[5] = new ModelMBeanOperationInfo("stop",  
			"stop(): start apache server",  
			stopParms, 
			"java.lang.Integer", 
			MBeanOperationInfo.ACTION, 
			stopDesc); 

		/* getters/setters for operations */
		//		MBeanParameterInfo[] getParms = new MBeanParameterInfo[0];

		Descriptor bytespsDesc =
			new DescriptorSupport(new String[] { "name=getBytesPerSecond",
			"descriptorType=operation", 
			"class=ApacheServer",
			"role=operation" }); 

		apacheOperations[6] =
			new ModelMBeanOperationInfo("getBytesPerSecond",  
				"number of bytes per second processed",  
				getParms, 
				"int", 
				MBeanOperationInfo.ACTION, 
				bytespsDesc); 

		// Define Notifications in ModelMBeanNotificationInfo
		// declare a Apache Server Down notification
		ModelMBeanNotificationInfo[] apacheNotifications =
			new ModelMBeanNotificationInfo[1];
			
		Descriptor apacheDownEventDesc =
			new DescriptorSupport(
				new String[] {
					"descriptorType=notification",
					"name=jmx.ModelMBean.General.Apache.Down",
					"severity=1",
					"MessageId=Apache001" });

		apacheNotifications[0] =
			new ModelMBeanNotificationInfo(
				new String[] { "jmx.ModelMBean.General.Apache.Down" },
				"jmx.ModelMBean.General",
				"Apache Server Down",
		// message text
				apacheDownEventDesc);

		// Create the ModelMBeanInfo		
		ModelMBeanInfo apacheMMBeanInfo =
			new ModelMBeanInfoSupport(
				"Apache",
				"ModelMBean for managing an Apache Web Server",
				apacheAttributes,
				apacheConstructors,
				apacheOperations,
				apacheNotifications);

		// Set the MBean's Descriptor for the ModelMBeanInfo
		try {
			apacheMMBeanInfo.setMBeanDescriptor(apacheDescription);
		} catch (Exception e) {
			System.out.println("CreateMBeanInfo failed with " + e.getMessage());
		}
		return apacheMMBeanInfo;
	}

}
