package net.jnjmx.ch4;

import java.util.SortedSet;
import java.util.TreeSet;
import java.lang.reflect.Constructor;
import javax.management.*;
import javax.management.modelmbean.*;

// Managed Server3 does not create ModelMbeans and ModelMBeanInfos at all
// In this example we show how to create the same ModelMBeans using an XMLService to 
// create the ModelMBeanInfo from an External program: toddManager.
//
// The main loop of the Server instantiates the server 
// The server's constructor instantiates the SessionPool 
//
// This illustrates how to create ModelMBeans for resources without changing the 
// implementation of the origional resource. 

public class Server3 {

	private SessionPool3 sessions;
	private SortedSet connectionQueue;
	private Listener listener;
	private boolean active;
	
	static boolean tracing = false;

	private long tzero;
	private int connections;
	
	// Instantiate Server3, No MBean is created
	public Server3() throws Exception {

		connectionQueue = new TreeSet();
		connections = 0;

		sessions = new SessionPool3();
  		  		
		active = true;
		tzero = System.currentTimeMillis();
		
		start();

		while (isActive()) {
			Connection k = waitForConnection();
			activateSession(k);
		}
		
	}

	public SessionPool3 getSessionPool() {
		return sessions;
	}

	public void activateSession(Connection k) {
		Session3 s;
		synchronized (sessions) {
			while ((s = sessions.reserve()) == null) {
				try {
					sessions.wait();
				} catch (InterruptedException x) {
					// see if a session is available
				}
			}
		}
		s.activate(k);
		connections++;
	}

  /** 
   * Connections attribute getter
   * @returns total number of connections handled
   */

	public Integer getConnections() {
		return new Integer(connections);
	}
	
  /**
   * Sessions attribute getter
   * @returns number of active sessions
   */

	public Integer getSessions() {
		int as = sessions.getAvailableSessions().intValue();
		int sz = sessions.getSize().intValue();
		return new Integer(sz - as);
	}
	
  /**
   * Uptime attribute getter
   * @returns number of milliseconds since the Server was started
   */

	public Long getUptime() {
		return new Long(System.currentTimeMillis() - tzero);
	}

	public boolean isActive() {
		return active;
	}
	
  /**
   * Shutdown the server killing the process and any active sessions
   */
	public void shutdown() {
		System.exit(0);
	}
	
   
  /**
   * 
   * Start a listener thread that will queue incoming connections
   */ 
	public void start() {
		listener = new Listener(connectionQueue);
		listener.start();
	}
	
  /**
   * Stop the Servers listener thread; active sessions continue to
   * handle requests
   */

	public void stop() {
		listener.stopListening();
	}

	public Connection waitForConnection() {
		Connection k;
		synchronized (connectionQueue) {
			while (connectionQueue.isEmpty()) {
				try {
					connectionQueue.wait();
				} catch (InterruptedException x) {
					// see if the queue is still empty
				}
			}
			k = (Connection) connectionQueue.first();
			connectionQueue.remove(k);
		}
		return k;
	}
	
}
