package net.jnjmx.ch9;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import java.lang.reflect.*;

import javax.management.*;

// Integrated Dynamic MBean Example

public class TackCatalogD implements Catalog, Serializable, DynamicMBean {

    private static final String FILEEXT = ".catalog";
	
    private final String name;

    private boolean autoCheckpoint = false;
    private Map categories;
    private List items;
    private int nextCatalogNo = 1;
    private int accesses=0, sales=0, inventorySize=0;

    public TackCatalogD(String inName) {
	this.name = inName;
	this.categories = new HashMap();
	this.items = new ArrayList();
    }
    public static TackCatalogD create(String inName) throws IOException {
	String cfn = inName + FILEEXT; 
	File cf = new File(cfn);
	if (cf.exists()) {
	    cf.delete();
	}

	TackCatalogD catalog = new TackCatalogD(inName);
	ObjectOutputStream db =
	    new ObjectOutputStream(new FileOutputStream(cf));
	db.writeObject(catalog);
	db.flush();
	db.close();

	return catalog;
    }
	
    // load is exposed in TackCatalogMBean

    public Catalog load(String name)
	throws IOException, ClassNotFoundException {
	String cfn = name + FILEEXT;
	File cf = new File(cfn);
	if (cf.exists() == false) {
	    throw new FileNotFoundException(cfn + " doesn't exist");
	}

	ObjectInputStream db = new ObjectInputStream(new FileInputStream(cf));
	TackCatalogD catalog = (TackCatalogD) db.readObject();
	db.close();

	return catalog;
    }


    public synchronized Map getCategories() {
	return this.categories;
    }



    public synchronized void classify(Item item, Category category) {
	List items = ((TackCategory) category).getItems();
	if (items.contains(item) == false) {
	    items.add(item);
	}
    }

    public synchronized Category createCategory(String name)
	throws CategoryExistsException {
	TackCategory result = new TackCategory(this, name);
	if (this.categories.get(name) != null) {
	    throw new CategoryExistsException();
	}
	this.categories.put(name, result);
	return result;
    }

    public synchronized Item createItem(String name, int price, int quantity)
	throws ItemExistsException {
	TackItem result = new TackItem(this, nextCatalogNo++, name, price, quantity);
	if (this.items.contains(result)) {
	    throw new ItemExistsException();
	}
	this.items.add(result);
	inventorySize = inventorySize + quantity;
	return result;
    }
	
    public synchronized void declassify(Item item, Category category) {
	List items = category.getItems();
	items.remove(item);
    }

    public synchronized void decreaseInventory(Item item, int amount)
	throws InventoryUnderflowException {
	inventorySize = inventorySize - amount;
	sales = sales + amount;
	int nq = item.getQuantity() - amount;
	if (nq < 0) {
	    throw new InventoryUnderflowException();
	}
	((TackItem)item).setQuantity(nq); 
    }

    public void destroy() throws IOException {
	String cfn = this.name + FILEEXT;
	File cf = new File(cfn);
	if (cf.exists()) {
	    cf.delete();
	}
    }

    public synchronized Category getCategory(String name) {
	Category result = (Category) this.categories.get(name);
	return result;
    }

    public synchronized void increaseInventory(Item item, int amount) {
	inventorySize = inventorySize + amount;
	if (amount < 0) {
	    throw new IllegalArgumentException();
	}
		
	int nq = item.getQuantity() + amount;
	((TackItem) item).setQuantity(nq);
    }



    public synchronized List getItems() {
	accesses++;
	return this.items;
    }

    public synchronized void removeCategory(Category category)
	throws NoSuchCategoryException {
	removeCategory(category.getName());
    }

    public synchronized void removeCategory(String name)
	throws NoSuchCategoryException {
	Category category = (Category) this.categories.remove(name);
	if (category == null) {
	    throw new NoSuchCategoryException();
	}
    }

    public synchronized void removeItem(Item item) throws NoSuchItemException {
	boolean exists = this.items.remove(item);
	if (exists == false) {
	    throw new NoSuchItemException();
	}
	inventorySize = inventorySize - item.getQuantity();
	removeItemFromCategories(item);
    }

    public synchronized void removeItem(int catalogno)
	throws NoSuchItemException {
	boolean removed = false;
	Iterator i = this.items.iterator();
	while (i.hasNext()) {
	    Item item = (Item) i.next();
	    if (item.getCatalogNo() == catalogno) {
		this.items.remove(item);
		inventorySize = inventorySize - item.getQuantity();
		removeItemFromCategories(item);
		removed = true;
		break;
	    }
	}
	if (removed == false) {
	    throw new NoSuchItemException();
	}	
    }
	
    private void removeItemFromCategories(Item item) {
	Iterator i = this.categories.keySet().iterator();
	while (i.hasNext()) {
	    String category = (String) i.next();
	    declassify(item, (Category) this.categories.get(category));
	}
    }
	
    // TackCatalogMBean Methods

    public boolean isAutoCheckpointing() {
	return this.autoCheckpoint;
    }
	
    public void setAutoCheckpoint(boolean autoCheckpoint) {
	this.autoCheckpoint = autoCheckpoint;
    }

    public void doCheckpoint() {
	if (this.autoCheckpoint == true) {
	    checkpoint();
	}
    }

    public void checkpoint() {
    }
	

    /* MBean Methods */
    public Object getAttribute(String attributeName) 
	throws AttributeNotFoundException,
	       MBeanException,
	       ReflectionException
    {
	if ( attributeName.equals("AutoCheckpointing") )
	    return new Boolean(autoCheckpoint);
	if ( attributeName.equals("NumberOfItems") )
	    return new Integer(items.size());
	if ( attributeName.equals("Accesses") )
	    return new Integer(accesses);
	if ( attributeName.equals("TotalSales") )
	    return new Integer(sales);
	if ( attributeName.equals("InventorySize") )
	    return new Integer(inventorySize);
	else throw(new AttributeNotFoundException("Attribute " + attributeName + " not found in " + this.getClass().getName()));
    }

    public AttributeList getAttributes(String[] attributes) {
	// in this simple example we always return all attributes
	// in a production example, you need to only return the
	// attributes in the list passed in
	AttributeList returnList = new AttributeList();
	if ( (attributes == null) || (attributes.length == 0) || (attributes.length > 0)) {
	    returnList.add(new Attribute("AutoCheckpointing",  new Boolean(autoCheckpoint)) );
	    returnList.add(new Attribute("NumberOfItems", new Integer(items.size())));
	    returnList.add(new Attribute("Accesses", new Integer(accesses)) );
	    returnList.add(new Attribute("TotalSales", new Integer(sales)) );
	    returnList.add(new Attribute("InventorySize", new Integer(inventorySize)) );
	}
	return returnList;

    }

    public void setAttribute(Attribute newAttribute) 
	throws AttributeNotFoundException,
	       InvalidAttributeValueException,
	       MBeanException, 
	       ReflectionException 
    {
	String attributeName = newAttribute.getName();
	if ( attributeName.equals("Autocheckpointing") )
	    autoCheckpoint = ((Boolean)(newAttribute.getValue())).booleanValue();
	else if ( (attributeName.equals("NumberOfItems")) || 
		  (attributeName.equals("TotalSales" )) ||
		  (attributeName.equals("Accesses" )) ||
		  (attributeName.equals("InventorySize")) ) {
	    throw(new AttributeNotFoundException("Attribute " + attributeName + " not updateable in " + this.getClass().getName())); 
	} else
	    throw(new AttributeNotFoundException("Attribute " + attributeName + " not found in " + this.getClass().getName()));

    }

    public AttributeList setAttributes(AttributeList newAttributes) {
	AttributeList returnList = new AttributeList();
	if ( newAttributes == null || newAttributes.isEmpty() )
	    return returnList;

	for ( Iterator I=newAttributes.iterator(); I.hasNext(); ) {
	    Attribute newAttribute= (Attribute)I.next();
	    try {
		setAttribute(newAttribute);
		returnList.add(new Attribute(newAttribute.getName(), 
					     newAttribute.getValue() ) );
	    } catch ( Exception e ) {
	    }

	    return returnList;
	}
	return returnList;
    }

    public Object invoke(String operationName, Object[] parms, String[] sig) {
	try {
	    if ( operationName.equals("doCheckpoint") ) {
		checkpoint();
	    } else if ( operationName.equals("load") ) {
		try {
		    return load((String)(parms[0]));
		}
		catch (Exception e) {
		    System.out.println("Load of TackCatalog failed");
		}
		return null;
	    } else {
		throw new 
		    ReflectionException(
					new NoSuchMethodException(operationName),
					"Cannot find the operation " + operationName + 
					" in TackCatalog");
	    }
	}
	catch (ReflectionException e) {
	    System.out.println("Load of TackCatalog failed");
	}
	return null;
    }

    public MBeanInfo getMBeanInfo() {
	/* Attributes: boolean AutoCheckpointing r/w, 
	   int numberOfItems r/o, 
	   int Accesses r/o, 
	   int Sales r/o 
	   int InventorySize r/o */
	/* operations: void saveCatalog, 
	   Catalog load, 
	   void doCheckpoint */
	/* notifications: none */
	String catDescription = "Tack Catalog Dynamic MBean";
	MBeanAttributeInfo[] catAttributes = new MBeanAttributeInfo[5];
	catAttributes[0] = new MBeanAttributeInfo("AutoCheckpointing",
						  "java.lang.Boolean",
						  "AutoCheckpointing: boolean value if checkpointing of catalog is done automatically",
						  true,true,false);


	catAttributes[1] = new MBeanAttributeInfo("NumberOfItems",
						  "java.lang.Integer",
						  "NumberOfItems: number of items in the catalog",
						  true,false,false);

	catAttributes[2] = new MBeanAttributeInfo("Accesses",
						  "java.lang.Integer",
						  "Accesses: number of time items in the catalog database have been listed",
						  true,false,false);

	catAttributes[3] = new MBeanAttributeInfo("TotalSales",
						  "java.lang.Integer",
						  "Sales: number of items sold",
						  true,false,false);
												  
	catAttributes[4] = new MBeanAttributeInfo("InventorySize",
						  "java.lang.Integer",
						  "InventorySize: Total number of all items currently in inventory",
						  true,true,false);

	MBeanConstructorInfo[] catConstructors = new MBeanConstructorInfo[2];
		
	Constructor[] constructors = this.getClass().getConstructors();
		
	catConstructors[0] = new MBeanConstructorInfo(
						      "tackCatalog(dbName): Constructs a tackCatalog from an existing database", 
						      constructors[0]);

	MBeanOperationInfo[] catOperations = new MBeanOperationInfo[2];
 
	MBeanParameterInfo[] params = new MBeanParameterInfo[0];

	catOperations[0] = new MBeanOperationInfo("doCheckpoint",
						  "doCheckpoint(): saves the catalog to a file",
						  params , 
						  "void", 
						  MBeanOperationInfo.ACTION);
	params =
	    new MBeanParameterInfo[] {
		(new MBeanParameterInfo("name",
					"java.lang.String",
					"Tack Catalog name"))};
				    
	catOperations[1] = new MBeanOperationInfo("load",
						  "load(dbName): loads the catalog from a file",
						  params , 
						  "void", 
						  MBeanOperationInfo.ACTION);
												  
	MBeanNotificationInfo[] catNotifications = new MBeanNotificationInfo[0];
		
	String className = this.getClass().getName();
		
	MBeanInfo catMBeanInfo = new MBeanInfo(className,
					       catDescription,
					       catAttributes,
					       catConstructors,
					       catOperations,
					       catNotifications );
									 
	return catMBeanInfo;
    } // getMBeanInfo

}	// class

