package net.jnjmx.ch9;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

// Tack Catalog Standard MBean Example
public class TackCatalogStandardMBeanExample implements Catalog, Serializable, TackCatalogMBean {

	private static final String FILEEXT = ".catalog";
	
	private final String name;

	private boolean autoCheckpoint = false;
	private Map categories;
	private List items;
	private int nextCatalogNo = 1;
	private int accesses=0, sales=0, inventorySize=0;

	public TackCatalogStandardMBeanExample(String inName) {
		this.name = inName;
		this.categories = new HashMap();
		this.items = new ArrayList();
	}
	public static TackCatalog create(String inName) throws IOException {
		String cfn = inName + FILEEXT; 
		File cf = new File(cfn);
		if (cf.exists()) {
			cf.delete();
		}

		TackCatalog catalog = new TackCatalog(inName);
		ObjectOutputStream db =
			new ObjectOutputStream(new FileOutputStream(cf));
		db.writeObject(catalog);
		db.flush();
		db.close();

		return catalog;
	}
	
	// load is exposed in TackCatalogMBean

	public TackCatalog load(String name)
		throws IOException, ClassNotFoundException {
		String cfn = name + FILEEXT;
		File cf = new File(cfn);
		if (cf.exists() == false) {
			throw new FileNotFoundException(cfn + " doesn't exist");
		}

		ObjectInputStream db = new ObjectInputStream(new FileInputStream(cf));
		TackCatalog catalog = (TackCatalog) db.readObject();
		db.close();

		return catalog;
	}


	public synchronized Map getCategories() {
		return this.categories;
	}



	public synchronized void classify(Item item, Category category) {
		List items = ((TackCategory) category).getItems();
		if (items.contains(item) == false) {
			items.add(item);
		}
	}

	public synchronized Category createCategory(String name)
		throws CategoryExistsException {
		TackCategory result = new TackCategory(this, name);
		if (this.categories.get(name) != null) {
			throw new CategoryExistsException();
		}
		this.categories.put(name, result);
		return result;
	}

	public synchronized Item createItem(String name, int price, int quantity)
		throws ItemExistsException {
		TackItem result = new TackItem(this, nextCatalogNo++, name, price, quantity);
		if (this.items.contains(result)) {
			throw new ItemExistsException();
		}
		this.items.add(result);
		inventorySize = inventorySize + quantity;
		return result;
	}
	
	public synchronized void declassify(Item item, Category category) {
		List items = category.getItems();
		items.remove(item);
	}

	public synchronized void decreaseInventory(Item item, int amount)
		throws InventoryUnderflowException {
		inventorySize = inventorySize - amount;
		sales = sales + amount;
		int nq = item.getQuantity() - amount;
		if (nq < 0) {
			throw new InventoryUnderflowException();
		}
		((TackItem)item).setQuantity(nq); 
	}

	public void destroy() throws IOException {
		String cfn = this.name + FILEEXT;
		File cf = new File(cfn);
		if (cf.exists()) {
			cf.delete();
		}
	}

	public synchronized Category getCategory(String name) {
		Category result = (Category) this.categories.get(name);
		return result;
	}

	public synchronized void increaseInventory(Item item, int amount) {
		inventorySize = inventorySize + amount;
		if (amount < 0) {
			throw new IllegalArgumentException();
		}
		
		int nq = item.getQuantity() + amount;
		((TackItem) item).setQuantity(nq);
	}



	public synchronized List getItems() {
		accesses++;
		return this.items;
	}

	public synchronized void removeCategory(Category category)
		throws NoSuchCategoryException {
		removeCategory(category.getName());
	}

	public synchronized void removeCategory(String name)
		throws NoSuchCategoryException {
		Category category = (Category) this.categories.remove(name);
		if (category == null) {
			throw new NoSuchCategoryException();
		}
	}

	public synchronized void removeItem(Item item) throws NoSuchItemException {
		boolean exists = this.items.remove(item);
		if (exists == false) {
			throw new NoSuchItemException();
		}
		inventorySize = inventorySize - item.getQuantity();
		removeItemFromCategories(item);
	}

	public synchronized void removeItem(int catalogno)
		throws NoSuchItemException {
		boolean removed = false;
		Iterator i = this.items.iterator();
		while (i.hasNext()) {
			Item item = (Item) i.next();
			if (item.getCatalogNo() == catalogno) {
				this.items.remove(item);
				inventorySize = inventorySize - item.getQuantity();
				removeItemFromCategories(item);
				removed = true;
				break;
			}
		}
		if (removed == false) {
			throw new NoSuchItemException();
		}	
	}
	
	private void removeItemFromCategories(Item item) {
		Iterator i = this.categories.keySet().iterator();
		while (i.hasNext()) {
			String category = (String) i.next();
			declassify(item, (Category) this.categories.get(category));
		}
	}
	
	// TackCatalogMBean Methods

	public boolean isAutoCheckpointing() {
		return this.autoCheckpoint;
	}
	
	public synchronized void setAutoCheckpoint(boolean autoCheckpoint) {
		this.autoCheckpoint = autoCheckpoint;
	}

	public void doCheckpoint() {
		if (this.autoCheckpoint == true) {
			checkpoint();
		}
	}

	public synchronized void checkpoint() {
	}
	
	public int getNumberOfItems() {
		return items.size();
	}
	
	public int getInventorySize() {
		return inventorySize;
	}
	
	public int getTotalSales() {
		return sales;
	}
	
	public int getAccesses() {
		return accesses;
	}
}
